//-----------------------------------------------------------------------------
// Main.c
// Blank program 
// Last UpDate 2010/01/10 15:00
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include "c8051f410.h"                 // SFR declarations
#include <stdio.h>

//-----------------------------------------------------------------------------
// Global CONSTANTS
//-----------------------------------------------------------------------------
#define SYSCLK       		24500000    // SYSCLK frequency in Hz
#define SPI_CLOCK   		800000      // Maximum SPI clock
#define BAUDRATE     		19200     	// Baud rate of UART in bps

#define MAX_BUFFER_SIZE    	8           // Maximum buffer Master will send
#define _CS					P0_3

//-----------------------------------------------------------------------------
// Global VARIAVBLES
//-----------------------------------------------------------------------------
long syscnt=0; //[@100315]
 

//-----------------------------------------------------------------------------
// Function PROTOTYPES
//-----------------------------------------------------------------------------
void putchar(char c);

void Init_Device (void);
void SYSCLK_Init (void);
void PCA0_Init (void);
void PORT_Init (void);
void UART0_Init (void);
void SPI0_Init (void);

unsigned char SPI_ByteRead(void);
unsigned char SPI_ByteWriteRead(unsigned char buf);
void Delay (int Delay_Count);


//-----------------------------------------------------------------------------
// MAIN Routine
//-----------------------------------------------------------------------------
void main (void) 
{
	unsigned char rdH = 0x00;
	unsigned char rdL = 0x00;

	// Init process
	Init_Device();
	_CS = 1;

	while (1)
	{
		_CS = 0;

		rdH = SPI_ByteWriteRead(0x00);
		rdL = SPI_ByteWriteRead(0x00);
		putchar(rdH);
		putchar(rdL);

		_CS = 1;

		putchar(0x0D);
		putchar(0x0A);

		Delay(30000);
   	}//	while (1)
}

//-----------------------------------------------------------------------------
// putchar
//-----------------------------------------------------------------------------
void putchar(char c) 
{ 
 	while(!TI);  
 	TI=0; 
 	SBUF = c; 
} 


//-----------------------------------------------------------------------------
// Initialization Subroutines
//-----------------------------------------------------------------------------
void Init_Device (void)
{
   PCA0_Init ();                       // Disable the Watchdog Timer first
   SYSCLK_Init ();
   PORT_Init ();

   SPI0_Init ();
   UART0_Init();                       // Initialize UART0 for printf's
   EA = 1;							   // enable global interrupts
}

//-----------------------------------------------------------------------------
// SYSCLK_Init
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// This routine initializes the system clock to use the internal 24.5MHz  
// oscillator as its clock source. Also enables missing clock detector reset.
//
//-----------------------------------------------------------------------------
void SYSCLK_Init (void)
{
   OSCICN = 0x87;                      // configure internal oscillator for
                                       // 24.5MHz 
   RSTSRC = 0x04;                      // enable missing clock detector
}

//-----------------------------------------------------------------------------
// PCA0_Init
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// This function disables the watchdog timer.
//
//-----------------------------------------------------------------------------
void PCA0_Init (void)
{
   PCA0MD &= ~0x40;                    // Disable the Watchdog Timer
   PCA0MD = 0x00;
}

//-----------------------------------------------------------------------------
// PORT_Init
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// Configure the Crossbar and GPIO ports.
// P0.4 - UART TX (push-pull)
// P0.5 - UART RX 
// P1.1 - ADC0 analog input
// P1.3 - LED (push-pull)
// 
//-----------------------------------------------------------------------------
void PORT_Init (void)
{
   	//P0MDOUT = 0x1D;                    	// Set P0.0,P0.2,P0.3,P0.4 to push-pull
	//P0SKIP 	= 0x00;

    P0MDOUT   = 0x01;
    P1MDOUT   = 0x80; 					// P1.7  -  Unassigned,  Push-Pull,  Digital [@100315]
	P1		  = 0x04;					// 010405 ǋL Vrefsgp̂
	P0_7      = 0;						// Initialize
	P0_6      = 0;						// Initialize
	P2_0      = 0;						// Initialize

    // P0.0  -  SCK  (SPI0), Push-Pull,  Digital
    // P0.1  -  MISO (SPI0), Open-Drain, Digital
    // P0.2  -  MOSI (SPI0), Push-Pull,  Digital
    // P0.3  -  NSS  (SPI0), Push-Pull,  Digital
    // P0.4  -  TX0 (UART0), Push-Pull,  Digital
    // P0.5  -  RX0 (UART0), Open-Drain, Digital
    // P0.6  -  Unassigned,  Open-Drain, Digital - Low
    // P0.7  -  Unassigned,  Open-Drain, Digital - Low

    // P1.0  -  Unassigned,  Open-Drain, Digital - Low
    // P1.1  -  Unassigned,  Open-Drain, Digital - Low
    // P1.2  -  Unassigned,  Open-Drain, Digital - Low
    // P1.3  -  Unassigned,  Open-Drain, Digital - Low
    // P1.4  -  Unassigned,  Open-Drain, Digital - Low
    // P1.5  -  Unassigned,  Open-Drain, Digital - Low
    // P1.6  -  Unassigned,  Open-Drain, Digital - Low
    // P1.7  -  Unassigned,  Push-Pull,  Digital

    // P2.0  -  Unassigned,  Open-Drain, Digital - Low
    // P2.1  -  Unassigned,  Open-Drain, Digital
    // P2.2  -  Unassigned,  Open-Drain, Digital
    // P2.3  -  Unassigned,  Open-Drain, Digital
    // P2.4  -  Unassigned,  Open-Drain, Digital
    // P2.5  -  Unassigned,  Open-Drain, Digital
    // P2.6  -  Unassigned,  Open-Drain, Digital
    // P2.7  -  Unassigned,  Open-Drain, Digital


   XBR0    = 0x03;                    // Enable UART0 and SPI
   XBR1    = 0x40;                    // Enable crossbar and Enable weak pull-ups

   //P1MDIN  &= 0xFB;					   // 010405ǋLF Vrefsgp̂
   //P1SKIP  |= 0x04;                    // 010405ǋLF Vrefsgp̂

}

//-----------------------------------------------------------------------------
// SPI0_Init
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// Configures SPI0 to use 4-wire Single Master mode. The SPI timing is
// configured for Mode 0,0 (data centered on first edge of clock phase and
// SCK line low in idle state).
//
//-----------------------------------------------------------------------------
void SPI0_Init()
{
   SPI0CFG   = 0x40;                   // Enable the SPI as a Master
                                       // CKPHA = '0', CKPOL = '0'
   SPI0CN    = 0x01;                   // 3-wire MOde, SPI enabled
 
   // SPI clock frequency equation from the datasheet
   SPI0CKR   = (SYSCLK/(2*SPI_CLOCK))-1;

   ESPI0 = 0;                          // Enable SPI interrupts
}

//-----------------------------------------------------------------------------
// UART0_Init
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// Configure the UART0 using Timer1, for <BAUDRATE> and 8-N-1.
//
//-----------------------------------------------------------------------------
void UART0_Init (void)
{
   SCON0 = 0x10;                       // SCON0: 8-bit variable bit rate
                                       //        level of STOP bit is ignored
                                       //        RX enabled
                                       //        ninth bits are zeros
                                       //        clear RI0 and TI0 bits
   if (SYSCLK/BAUDRATE/2/256 < 1) {
      TH1 = -(SYSCLK/BAUDRATE/2);
      CKCON |=  0x08;                  // T1M = 1; SCA1:0 = xx
   } else if (SYSCLK/BAUDRATE/2/256 < 4) {
      TH1 = -(SYSCLK/BAUDRATE/2/4);
      CKCON &= ~0x0B;                  // T1M = 0; SCA1:0 = 01
      CKCON |=  0x01;
   } else if (SYSCLK/BAUDRATE/2/256 < 12) {
      TH1 = -(SYSCLK/BAUDRATE/2/12);
      CKCON &= ~0x0B;                  // T1M = 0; SCA1:0 = 00
   } else if (SYSCLK/BAUDRATE/2/256 < 48) {
      TH1 = -(SYSCLK/BAUDRATE/2/48);
      CKCON &= ~0x0B;                  // T1M = 0; SCA1:0 = 10
      CKCON |=  0x02;
   } else {
      while (1);                       // Error.  Unsupported baud rate
   }

   TL1 = TH1;                          // init Timer1
   TMOD &= ~0xf0;                      // TMOD: timer 1 in 8-bit autoreload
   TMOD |=  0x20;
   TR1 = 1;                            // START Timer1
   TI0 = 1;                            // Indicate TX0 ready
}

//-----------------------------------------------------------------------------
// Interrupt Service Routines
//-----------------------------------------------------------------------------
//void Timer2_ISR (void) interrupt 5
//{

//}



//-----------------------------------------------------------------------------
// Interrupt Service Routines
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// SPI_ISR
//-----------------------------------------------------------------------------
//
// Handles all error checks and single-byte writes.
//
//-----------------------------------------------------------------------------
//void SPI_ISR (void) interrupt 6
//{


//}

unsigned char SPI_ByteWriteRead(unsigned char buf)
{
	unsigned char readbuf;

	SPI0DAT = buf;								// Write data to transfer buffer 
	while (!(SPI0CN & (1 << 7)));              	// Wait until the SPI transfer end; 
	readbuf = SPI0DAT;                        	// Read Data  
	SPI0CN &= ~0x80;                   		 	// Crear SPI Interrupt flag;

	return readbuf;
}

unsigned char SPI_ByteRead(void)
{
	unsigned char readbuf;

	readbuf = SPI0DAT;                        	// Read Data  
	SPI0CN &= ~0x80;                   		 	// Crear SPI Interrupt flag;

	return readbuf;
}


void Delay (int Delay_Count)
{
   unsigned long count;

   for (count = Delay_Count; count > 0; count--);
}
//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------
